-- plugin/gemini/api.lua

local M = {}

local function get_api_key()
	-- Check for environment variable
	local api_key = os.getenv("GEMINI_API_KEY")
	if api_key then
		return api_key
	end

	-- Check for Neovim global variable
	api_key = vim.g.gemini_api_key
	if api_key then
		return api_key
	end

	return nil -- API key not found
end

local function make_request(prompt)
	local api_key = get_api_key() -- Get API key from environment or global variable

	if not api_key then
		vim.notify(
			"Google AI API key not set. Set GEMINI_API_KEY environment variable or g.gemini_api_key in your init.vim or init.lua.",
			vim.log.levels.ERROR
		)
		return nil
	end

	local socket = require("socket")
	local ssl = require("ssl")
	local https = require("socket.https")

	local model = "gemini-2.0-flash" -- SPECIFY Gemini 2.0 Flash MODEL

	local url =
		string.format("https://generative-ai.googleapis.com/v1beta/models/%s:generateContent?key=%s", model, api_key)

	-- Construct the JSON payload
	local payload = vim.json.encode({
		contents = {
			{
				parts = {
					{
						text = prompt,
					},
				},
			},
		},
	})

	local body, code, headers, status = https.request({
		url = url,
		method = "POST",
		headers = {
			["Content-Type"] = "application/json",
		},
		source = socket.source.string(payload),
		protocol = "tlsv1_2", -- Require TLS 1.2
	})

	if code ~= 200 then
		vim.notify(string.format("HTTP request failed with code %d: %s", code, status), vim.log.levels.ERROR)
		return nil
	end

	local decoded_result = vim.json.decode(body)

	return decoded_result
end

function M.get_response(prompt)
	local result = make_request(prompt)

	if
		result
		and result.candidates
		and result.candidates[1]
		and result.candidates[1].content
		and result.candidates[1].content.parts
	then
		return result.candidates[1].content.parts[1].text
	else
		vim.notify("No response from Google AI API or malformed response.", vim.log.levels.ERROR)
		return nil
	end
end

print("api.lua loaded")

return M
